@@ -23,6 +23,7 @@
 
 #include <config.h>
 
+#include <string.h>
 #include <limits.h>
 #include <util/log.h>
 
@@ -35,6 +36,7 @@ typedef struct {
     JSRuntime *runtime;
     JSContext *context;
     JSObject *obj;
+    guint unref_on_global_object_finalized : 1;
 } Closure;
 
 /*
@@ -62,7 +64,7 @@ typedef struct {
  * the closure if it isn't.
  *
  * The closure can thus be destroyed in several cases:
- * - invalidation by say signal disconnection; we get invalidate callback
+ * - invalidation by unref, e.g. when a signal is disconnected, closure is unref'd
  * - invalidation because we were invoked while the context was dead
  * - invalidation through finalization (we were garbage collected)
  *
@@ -81,9 +83,8 @@ invalidate_js_pointers(Closure *c)
     c->context = NULL;
     c->runtime = NULL;
 
-    /* disconnects from signals, for example...
-     * potentially a dangerous re-entrancy but
-     * we'll have to risk it.
+    /* Notify any closure reference holders they
+     * may want to drop references.
      */
     g_closure_invalidate(&c->base);
 }
@@ -93,12 +94,19 @@ global_context_finalized(JSObject *obj,
                          void     *data)
 {
     Closure *c;
+    gboolean need_unref;
 
     c = data;
 
-    gjs_debug(GJS_DEBUG_GCLOSURE,
-              "Context destroy notifier on closure %p which calls object %p",
-              c, c->obj);
+    gjs_debug_closure("Context global object destroy notifier on closure %p "
+                      "which calls object %p",
+                      c, c->obj);
+
+    /* invalidate_js_pointers() could free us so check flag now to avoid
+     * invalid memory access
+     */
+    need_unref = c->unref_on_global_object_finalized;
+    c->unref_on_global_object_finalized = FALSE;
 
     if (c->obj != NULL) {
         g_assert(c->obj == obj);
@@ -106,11 +114,11 @@ global_context_finalized(JSObject *obj,
         invalidate_js_pointers(c);
     }
 
-    /* The "Keep Alive" (garbage collector) owns one reference. */
-    g_closure_unref(&c->base);
+    if (need_unref) {
+        g_closure_unref(&c->base);
+    }
 }
 
-
 static void
 check_context_valid(Closure *c)
 {
@@ -128,16 +136,22 @@ check_context_valid(Closure *c)
         }
     }
 
-    gjs_debug(GJS_DEBUG_GCLOSURE,
-              "Context %p no longer exists, invalidating closure %p which calls object %p",
-              c->context, c, c->obj);
+    gjs_debug_closure("Context %p no longer exists, invalidating "
+                      "closure %p which calls object %p",
+                      c->context, c, c->obj);
 
     /* Did not find the context. */
     invalidate_js_pointers(c);
 }
 
-/* Invalidation is like "dispose" - it happens on signal disconnect,
- * is guaranteed to happen at finalize, but may happen before finalize
+/* Invalidation is like "dispose" - it is guaranteed to happen at
+ * finalize, but may happen before finalize. Normally, g_closure_invalidate()
+ * is called when the "target" of the closure becomes invalid, so that the
+ * source (the signal connection, say can be removed.) The usage above
+ * in invalidate_js_pointers() is typical. Since the target of the closure
+ * is under our control, it's unlikely that g_closure_invalidate() will ever
+ * be called by anyone else, but in case it ever does, it's slightly better
+ * to remove the "keep alive" here rather than in the finalize notifier.
  */
 static void
 closure_invalidated(gpointer data,
@@ -147,25 +161,70 @@ closure_invalidated(gpointer data,
 
     c = (Closure*) closure;
 
-    gjs_debug(GJS_DEBUG_GCLOSURE,
-              "Invalidating closure %p which calls object %p",
-              closure, c->obj);
+    gjs_debug_closure("Invalidating closure %p which calls object %p",
+                      closure, c->obj);
 
-    if (c->obj) {
+    if (c->obj == NULL) {
+        gjs_debug_closure("   (closure %p already dead, nothing to do)",
+                          closure);
+        return;
+    }
+
+    /* this will set c->obj to null if the context is dead
+     */
+    check_context_valid(c);
+
+    if (c->obj == NULL) {
+        /* Context is dead here. This happens if, as a side effect of
+         * tearing down the context, the closure was invalidated,
+         * say be some other finalized object that had a ref to
+         * the closure dropping said ref.
+         *
+         * Because c->obj was not NULL at the start of
+         * closure_invalidated, we know that
+         * global_context_finalized() has not been called.  So we know
+         * we are not being invalidated from inside
+         * global_context_finalized().
+         *
+         * That means global_context_finalized() has yet to be called,
+         * but we know it will be called, because the context is dead
+         * and thus its global object should be finalized.
+         *
+         * We can't call gjs_keep_alive_remove_global_child() because
+         * the context is invalid memory and we can't get to the
+         * global object that stores the keep alive.
+         *
+         * So global_context_finalized() could be called on an
+         * already-finalized closure. To avoid this, we temporarily
+         * ref ourselves, and set a flag to remove this ref
+         * in global_context_finalized().
+         */
+        gjs_debug_closure("   (closure %p's context was dead, holding ref "
+                          "until global object finalize)",
+                          closure);
+
+        c->unref_on_global_object_finalized = TRUE;
+        g_closure_ref(&c->base);
+    } else {
+        /* If the context still exists, then remove our destroy
+         * notifier.  Otherwise we would call the destroy notifier on
+         * an already-freed closure.
+         *
+         * This happens in the normal case, when the closure is
+         * invalidated for some reason other than destruction of the
+         * JSContext.
+         */
+        gjs_debug_closure("   (closure %p's context was alive, "
+                          "removing our destroy notifier on global object)",
+                          closure);
         gjs_keep_alive_remove_global_child(c->context,
-                                              global_context_finalized,
-                                              c->obj,
-                                              c);
+                                           global_context_finalized,
+                                           c->obj,
+                                           c);
 
         c->obj = NULL;
         c->context = NULL;
         c->runtime = NULL;
-
-        /* The "Keep Alive" (garbage collector) owns one reference,
-         * since we removed ourselves from the keep-alive we'll
-         * never be collected so drop the ref here
-         */
-        g_closure_unref(&c->base);
     }
 }
 
@@ -197,8 +256,8 @@ gjs_closure_invoke(GClosure *closure,
     }
 
     if (JS_IsExceptionPending(context)) {
-        gjs_debug(GJS_DEBUG_GCLOSURE,
-                  "Exception was pending before invoking callback??? Not expected");
+        gjs_debug_closure("Exception was pending before invoking callback??? "
+                          "Not expected");
         gjs_log_exception(c->context, NULL);
     }
 
@@ -209,21 +268,54 @@ gjs_closure_invoke(GClosure *closure,
                                  argv,
                                  retval)) {
         /* Exception thrown... */
-        gjs_debug(GJS_DEBUG_GCLOSURE,
-                  "Closure invocation failed (exception should have been thrown) closure %p callable %p",
-                  closure, c->obj);
+        gjs_debug_closure("Closure invocation failed (exception should "
+                          "have been thrown) closure %p callable %p",
+                          closure, c->obj);
         if (!gjs_log_exception(context, NULL))
-            gjs_debug(GJS_DEBUG_ERROR,
-                      "Closure invocation failed but no exception was set?");
+            gjs_debug_closure("Closure invocation failed but no exception was set?");
         return;
     }
 
     if (gjs_log_exception(context, NULL)) {
-        gjs_debug(GJS_DEBUG_ERROR,
-                  "Closure invocation succeeded but an exception was set");
+        gjs_debug_closure("Closure invocation succeeded but an exception was set");
     }
 }
 
+gboolean
+gjs_closure_invoke_simple(JSContext   *context,
+                          GClosure    *closure,
+                          jsval       *retval,
+                          const gchar *format,
+                          ...)
+{
+    va_list ap;
+    int argc;
+    void *stack_space;
+    jsval *argv;
+    int i;
+
+    va_start(ap, format);
+    argv = JS_PushArgumentsVA(context, &stack_space, format, ap);
+    va_end(ap);
+    if (!argv)
+        return FALSE;
+
+    argc = (int)strlen(format);
+    for (i = 0; i < argc; i++)
+        JS_AddRoot(context, &argv[i]);
+    JS_AddRoot(context, retval);
+
+    gjs_closure_invoke(closure, argc, argv, retval);
+
+    for (i = 0; i < argc; i++)
+        JS_RemoveRoot(context, &argv[i]);
+    JS_RemoveRoot(context, retval);
+
+    JS_PopArguments(context, stack_space);
+
+    return TRUE;
+}
+
 JSContext*
 gjs_closure_get_context(GClosure *closure)
 {
@@ -261,6 +353,7 @@ gjs_closure_new(JSContext  *context,
      */
     c->context = gjs_runtime_get_load_context(c->runtime);
     c->obj = callable;
+    c->unref_on_global_object_finalized = FALSE;
 
     GJS_INC_COUNTER(closure);
     /* the finalize notifier right now is purely to track the counter
@@ -273,14 +366,11 @@ gjs_closure_new(JSContext  *context,
                                     c->obj,
                                     c);
 
-    /* The "Keep Alive" (garbage collector) owns one reference. */
-    g_closure_ref(&c->base);
-
     g_closure_add_invalidate_notifier(&c->base, NULL, closure_invalidated);
 
-    gjs_debug(GJS_DEBUG_GCLOSURE,
-              "Create closure %p which calls object %p '%s'",
-              c, c->obj, description);
+    gjs_debug_closure("Create closure %p which calls object %p '%s'",
+                      c, c->obj, description);
 
     return &c->base;
 }
+
