@@ -1,6 +1,6 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (c) 2008  LiTL, LLC
+ * Copyright (c) 2008  litl, LLC
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -28,13 +28,13 @@
 #include "function.h"
 #include "object.h"
 #include "boxed.h"
+#include "union.h"
 #include "enumeration.h"
 #include "arg.h"
 
 #include <gjs/mem.h>
 
 #include <util/log.h>
-#include <util/dirs.h>
 #include <util/misc.h>
 
 #include <jsapi.h>
@@ -58,25 +58,39 @@ resolve_namespace_object(JSContext  *context,
 {
     GIRepository *repo;
     GError *error;
-    char *fixed_ns_name;
+    JSContext *load_context;
+    jsval versions_val;
+    JSObject *versions;
+    jsval version_val;
+    const char *version;
 
-    fixed_ns_name = gjs_fix_ns_name(ns_name);
+    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
+    if (!gjs_object_require_property(load_context, repo_obj, "GI repository object", "versions", &versions_val) ||
+        !JSVAL_IS_OBJECT(versions_val)) {
+        gjs_throw(context, "No 'versions' property in GI repository object");
+        return NULL;
+    }
+
+    versions = JSVAL_TO_OBJECT(versions_val);
+
+    version = NULL;
+    if (JS_GetProperty(load_context, versions, ns_name, &version_val) &&
+        JSVAL_IS_STRING(version_val)) {
+        version = gjs_string_get_ascii(version_val);
+    }
 
     repo = g_irepository_get_default();
 
     error = NULL;
-    g_irepository_require(repo, fixed_ns_name, 0, &error);
+    g_irepository_require(repo, ns_name, version, 0, &error);
     if (error != NULL) {
         gjs_throw(context,
-                  "Requiring %s fixed as %s: %s",
-                  ns_name, fixed_ns_name, error->message);
+                  "Requiring %s, version %s: %s",
+                  ns_name, version?version:"none", error->message);
         g_error_free(error);
-        g_free(fixed_ns_name);
         return JS_FALSE;
     }
 
-    g_free(fixed_ns_name);
-
     /* Defines a property on "obj" (the javascript repo object)
      * with the given namespace name, pointing to that namespace
      * in the repo.
@@ -218,6 +232,7 @@ repo_new(JSContext *context)
 {
     JSObject *repo;
     JSObject *global;
+    JSObject *versions;
 
     /* We have to define the class in the global object so we can JS_ConstructObject */
 
@@ -262,6 +277,16 @@ repo_new(JSContext *context)
         return JS_FALSE;
     }
 
+    versions = JS_ConstructObject(context, NULL, NULL, NULL);
+
+    JS_DefineProperty(context, repo,
+                      "versions",
+                      OBJECT_TO_JSVAL(versions),
+                      NULL, NULL,
+                      JSPROP_PERMANENT);
+
+    g_assert(gjs_object_has_property(context, repo, "versions"));
+
     /* FIXME - hack to make namespaces load, since
      * gobject-introspection does not yet search a path properly.
      */
@@ -280,14 +305,6 @@ gjs_define_repo(JSContext  *context,
 {
     JSObject *repo;
 
-    if (gjs_environment_variable_is_set("GJS_USE_UNINSTALLED_FILES")) {
-        const char *builddir = g_getenv("BUILDDIR");
-
-        if (builddir != NULL) {
-            g_irepository_prepend_search_path(builddir);
-        }
-    }
-
     repo = repo_new(context);
 
     if (!JS_DefineProperty(context, module_obj,
@@ -312,7 +329,7 @@ gjs_define_constant(JSContext      *context,
     type_info = g_constant_info_get_type(info);
     g_constant_info_get_value(info, &garg);
 
-    if (!gjs_value_from_g_arg(context, &value, type_info, &garg)) {
+    if (!gjs_value_from_g_argument(context, &value, type_info, &garg)) {
         g_base_info_unref((GIBaseInfo*) type_info);
         return JS_FALSE;
     }
@@ -404,7 +421,6 @@ gjs_define_info(JSContext  *context,
 #if GJS_VERBOSE_ENABLE_GI_USAGE
     _gjs_log_info_usage(info);
 #endif
-
     switch (g_base_info_get_type(info)) {
     case GI_INFO_TYPE_FUNCTION:
         {
@@ -415,13 +431,25 @@ gjs_define_info(JSContext  *context,
         }
         break;
     case GI_INFO_TYPE_OBJECT:
-        if (!gjs_define_object_class(context, in_object, G_TYPE_INVALID, (GIObjectInfo*) info, NULL, NULL))
-            return JS_FALSE;
+        {
+            GType gtype;
+            GIBaseInfo *info_for_gtype;
+            gtype = g_registered_type_info_get_g_type((GIRegisteredTypeInfo*)info);
+            if (!gjs_define_object_class(context, in_object, gtype, NULL, NULL, &info_for_gtype))
+                return JS_FALSE;
+            g_assert(g_base_info_equal(info, info_for_gtype));
+            g_base_info_unref(info_for_gtype);
+        }
         break;
+    case GI_INFO_TYPE_STRUCT:
     case GI_INFO_TYPE_BOXED:
         if (!gjs_define_boxed_class(context, in_object, (GIBoxedInfo*) info, NULL, NULL))
             return JS_FALSE;
         break;
+    case GI_INFO_TYPE_UNION:
+        if (!gjs_define_union_class(context, in_object, (GIUnionInfo*) info, NULL, NULL))
+            return JS_FALSE;
+        break;
     case GI_INFO_TYPE_ENUM:
     case GI_INFO_TYPE_FLAGS:
         if (!gjs_define_enumeration(context, in_object, (GIEnumInfo*) info, NULL))
@@ -481,7 +509,7 @@ gjs_lookup_namespace_object_by_name(JSContext      *context,
     global = JS_GetGlobalObject(load_context);
 
     importer = JSVAL_VOID;
-    if (!gjs_object_require_property(load_context, global, "imports", &importer) ||
+    if (!gjs_object_require_property(load_context, global, "global object", "imports", &importer) ||
         !JSVAL_IS_OBJECT(importer)) {
         gjs_log_exception(load_context, NULL);
         gjs_throw(context, "No imports property in global object");
@@ -489,7 +517,7 @@ gjs_lookup_namespace_object_by_name(JSContext      *context,
     }
 
     girepository = JSVAL_VOID;
-    if (!gjs_object_require_property(load_context, JSVAL_TO_OBJECT(importer),
+    if (!gjs_object_require_property(load_context, JSVAL_TO_OBJECT(importer), "importer",
                                         "gi", &girepository) ||
         !JSVAL_IS_OBJECT(girepository)) {
         gjs_log_exception(load_context, NULL);
@@ -499,7 +527,7 @@ gjs_lookup_namespace_object_by_name(JSContext      *context,
 
     repo_obj = JSVAL_TO_OBJECT(girepository);
 
-    if (!gjs_object_require_property(context, repo_obj, ns, &ns_obj))
+    if (!gjs_object_require_property(context, repo_obj, "GI repository object", ns, &ns_obj))
         return NULL;
 
     if (!JSVAL_IS_OBJECT(ns_obj)) {
