@@ -41,9 +41,8 @@ gjs_js_one_value_from_dbus(JSContext       *context,
 
     arg_type = dbus_message_iter_get_arg_type(iter);
 
-    gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-              "Converting dbus type '%c' to jsval",
-              arg_type != DBUS_TYPE_INVALID ? arg_type : '0');
+    gjs_debug_dbus_marshal("Converting dbus type '%c' to jsval",
+                           arg_type != DBUS_TYPE_INVALID ? arg_type : '0');
 
     switch (arg_type) {
     case DBUS_TYPE_STRUCT:
@@ -99,27 +98,39 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                 dbus_message_iter_recurse(iter, &array_iter);
                 while (dbus_message_iter_get_arg_type(&array_iter) != DBUS_TYPE_INVALID) {
                     DBusMessageIter entry_iter;
+                    jsval key_value, entry_value;
+                    JSString *key_str;
                     const char *key;
-                    jsval entry_value;
 
                     dbus_message_iter_recurse(&array_iter, &entry_iter);
 
-                    if (dbus_message_iter_get_arg_type(&entry_iter) != DBUS_TYPE_STRING) {
-                        gjs_throw(context, "Dictionary keys are not strings, can't convert to JavaScript");
+                    if (!dbus_type_is_basic(dbus_message_iter_get_arg_type(&entry_iter))) {
+                        gjs_throw(context, "Dictionary keys are not a basic type, can't convert to JavaScript");
                         JS_RemoveRoot(context, &obj);
                         return JS_FALSE;
                     }
 
-                    dbus_message_iter_get_basic(&entry_iter, &key);
+                    entry_value = JSVAL_VOID;
+                    JS_AddRoot(context, &entry_value);
+                    if (!gjs_js_one_value_from_dbus(context, &entry_iter, &key_value)) {
+                        JS_RemoveRoot(context, &key_value);
+                        JS_RemoveRoot(context, &obj);
+                        return JS_FALSE;
+                    }
+
+                    key_str = JS_ValueToString(context, key_value);
+                    JS_AddRoot(context, &key_str);
+                    key = JS_GetStringBytes(key_str);
 
                     dbus_message_iter_next(&entry_iter);
 
-                    gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                              "Defining dict entry %s in jsval dict", key);
+                    gjs_debug_dbus_marshal("Defining dict entry %s in jsval dict", key);
 
                     entry_value = JSVAL_VOID;
                     JS_AddRoot(context, &entry_value);
                     if (!gjs_js_one_value_from_dbus(context, &entry_iter, &entry_value)) {
+                        JS_RemoveRoot(context, &key_value);
+                        JS_RemoveRoot(context, &key_str);
                         JS_RemoveRoot(context, &entry_value);
                         JS_RemoveRoot(context, &obj);
                         return JS_FALSE;
@@ -128,11 +139,15 @@ gjs_js_one_value_from_dbus(JSContext       *context,
                     if (!JS_DefineProperty(context, obj,
                                            key, entry_value,
                                            NULL, NULL, JSPROP_ENUMERATE)) {
+                        JS_RemoveRoot(context, &key_value);
+                        JS_RemoveRoot(context, &key_str);
                         JS_RemoveRoot(context, &entry_value);
                         JS_RemoveRoot(context, &obj);
                         return JS_FALSE;
                     }
 
+                    JS_RemoveRoot(context, &key_value);
+                    JS_RemoveRoot(context, &key_str);
                     JS_RemoveRoot(context, &entry_value);
                     dbus_message_iter_next(&array_iter);
                 }
@@ -603,8 +618,7 @@ append_array(JSContext         *context,
                 return JS_FALSE;
             }
 
-            gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                      " Adding struct element %u", i);
+            gjs_debug_dbus_marshal(" Adding struct element %u", i);
 
             if (!gjs_js_one_value_to_dbus(context, element, &struct_iter,
                                           &element_sig_iter))
@@ -639,8 +653,7 @@ append_array(JSContext         *context,
                 return JS_FALSE;
             }
 
-            gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                      " Adding array element %u", i);
+            gjs_debug_dbus_marshal(" Adding array element %u", i);
 
             if (!gjs_js_one_value_to_dbus(context, element, &array_iter,
                                           &element_sig_iter))
@@ -802,9 +815,8 @@ append_dict(JSContext         *context,
         if (!gjs_object_require_property(context, props, "DBus append_dict", name, &propval))
             return JS_FALSE;
 
-        gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                  " Adding property %s",
-                  name);
+        gjs_debug_dbus_marshal(" Adding property %s",
+                               name);
 
         /* gjs_js_one_value_to_dbus() would check this also, but would not
          * print the property name, which is often useful
@@ -869,9 +881,8 @@ gjs_js_one_value_to_dbus(JSContext         *context,
 
     forced_type = dbus_signature_iter_get_current_type(sig_iter);
 
-    gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-              "Converting dbus type '%c' from jsval",
-              forced_type != DBUS_TYPE_INVALID ? forced_type : '0');
+    gjs_debug_dbus_marshal("Converting dbus type '%c' from jsval",
+                           forced_type != DBUS_TYPE_INVALID ? forced_type : '0');
 
     /* Don't write anything on the bus if the signature is empty */
     if (forced_type == DBUS_TYPE_INVALID)
@@ -955,13 +966,11 @@ gjs_js_one_value_to_dbus(JSContext         *context,
 
             length = JSVAL_TO_INT(lengthval);
 
-            gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                              "Looks like an array length %u", length);
+            gjs_debug_dbus_marshal("Looks like an array length %u", length);
             if (!append_array(context, iter, sig_iter, obj, length))
                 return JS_FALSE;
         } else {
-            gjs_debug(GJS_DEBUG_DBUS_MARSHAL,
-                      "Looks like a dictionary");
+            gjs_debug_dbus_marshal("Looks like a dictionary");
             if (!append_dict(context, iter, sig_iter, obj))
                 return JS_FALSE;
         }
@@ -1019,26 +1028,3 @@ gjs_js_values_to_dbus(JSContext         *context,
     return JS_TRUE;
 }
 
-/* If jsval is an object, add properties from the DBusMessage such as the
- * sender. If jsval is not an object, do nothing.
- */
-JSBool
-gjs_js_add_dbus_props(JSContext       *context,
-                      DBusMessage     *message,
-                      jsval            value)
-{
-    const char *sender;
-
-    if (!JSVAL_IS_OBJECT(value))
-        return JS_TRUE;
-
-    sender = dbus_message_get_sender(message);
-
-    if (!JS_DefineProperty(context, JSVAL_TO_OBJECT(value),
-                           "_dbus_sender",
-                           STRING_TO_JSVAL(JS_NewStringCopyZ(context, sender)),
-                           NULL, NULL, JSPROP_ENUMERATE))
-        return JS_FALSE;
-
-    return JS_TRUE;
-}
