@@ -93,12 +93,28 @@ is_allowed_prefix (const char *prefix)
 
 #define PREFIX_LENGTH 12
 
+static void
+write_to_stream(FILE       *logfp,
+                gboolean    error,
+                const char *prefix,
+                const char *s)
+{
+    /* seek to end to avoid truncating in case we're using shared logfile */
+    (void)fseek(logfp, 0, SEEK_END);
+
+    fprintf(logfp, "%*s: %s%s", PREFIX_LENGTH, prefix, error ? "!!!   " : "", s);
+    if (!g_str_has_suffix(s, "\n"))
+        fputs("\n", logfp);
+    fflush(logfp);
+}
+
 void
 gjs_debug(GjsDebugTopic topic,
           const char   *format,
           ...)
 {
     static FILE *logfp = NULL;
+    static gboolean debug_log_enabled = FALSE;
     static gboolean strace_timestamps = FALSE;
     static gboolean checked_for_timestamp = FALSE;
     static gboolean print_timestamp = FALSE;
@@ -119,7 +135,10 @@ gjs_debug(GjsDebugTopic topic,
 
     if (logfp == NULL) {
         const char *debug_output = g_getenv("GJS_DEBUG_OUTPUT");
-        if (debug_output != NULL) {
+        if (debug_output != NULL &&
+            strcmp(debug_output, "stderr") == 0) {
+            debug_log_enabled = TRUE;
+        } else if (debug_output != NULL) {
             const char *log_file;
             char *free_me;
             char *c;
@@ -146,6 +165,8 @@ gjs_debug(GjsDebugTopic topic,
                         log_file, g_strerror(errno));
 
             g_free(free_me);
+
+            debug_log_enabled = TRUE;
         }
 
         if (logfp == NULL)
@@ -154,6 +175,14 @@ gjs_debug(GjsDebugTopic topic,
         strace_timestamps = gjs_environment_variable_is_set("GJS_STRACE_TIMESTAMPS");
     }
 
+    /* only log errors and strace timestamps if debug
+     * log wasn't specifically switched on
+     */
+    if ((!debug_log_enabled) &&
+        !(topic == GJS_DEBUG_ERROR ||
+          topic == GJS_DEBUG_STRACE_TIMESTAMP))
+        return;
+
     error = FALSE;
     prefix = "???";
     switch (topic) {
@@ -194,6 +223,9 @@ gjs_debug(GjsDebugTopic topic,
     case GJS_DEBUG_DBUS:
         prefix = "JS DBUS";
         break;
+    case GJS_DEBUG_DBUS_MARSHAL:
+        prefix = "JS DBUS MARSHAL";
+        break;
     case GJS_DEBUG_KEEP_ALIVE:
         prefix = "JS KP ALV";
         break;
@@ -259,30 +291,38 @@ gjs_debug(GjsDebugTopic topic,
         access(s2, F_OK);
         g_free(s2);
     } else {
-        /* seek to end to avoid truncating in case we're using shared logfile */
-        (void)fseek(logfp, 0, SEEK_END);
-
         if (print_timestamp) {
             static gdouble previous = 0.0;
             gdouble total = g_timer_elapsed(timer, NULL) * 1000.0;
             gdouble since = total - previous;
+            const char *ts_suffix;
+            char *s2;
 
-            fprintf(logfp, "%g ", total);
             if (since > 50.0) {
-                fprintf(logfp, "!!  ");
+                ts_suffix = "!!  ";
             } else if (since > 100.0) {
-                fprintf(logfp, "!!! ");
+                ts_suffix = "!!! ";
             } else if (since > 200.0) {
-                fprintf(logfp, "!!!!");
+                ts_suffix = "!!!!";
             } else {
-                fprintf(logfp, "    ");
+                ts_suffix = "    ";
             }
+
+            s2 = g_strdup_printf("%g %s%s",
+                                 total, ts_suffix, s);
+            g_free(s);
+            s = s2;
+
             previous = total;
         }
-        fprintf(logfp, "%*s: %s%s", PREFIX_LENGTH, prefix, error ? "!!!   " : "", s);
-        if (!g_str_has_suffix(s, "\n"))
-            fputs("\n", logfp);
-        fflush(logfp);
+
+        /* write to both stderr and logfile
+         * if it's an error
+         */
+        if (error && logfp != stderr)
+            write_to_stream(stderr, error, prefix, s);
+
+        write_to_stream(logfp, error, prefix, s);
     }
 
     g_free(s);
